from __future__ import absolute_import, unicode_literals
from ._compat import range, open
from .utils import log
import re
import os

def graph(visitor, block):
    """
    Format:
        
        {% graph %}
        graph TD;
            A-->B;
            A-->C;
            B-->D;
            C-->D;
        {% endgraph %}
        
    """
    from .utils import json_dumps
    
    txt = []
    txt.append('<div class="mermaid">')
    txt.append(block['body'])
    txt.append('</div>')
    return '\n'.join(txt)




from __future__ import print_function
from ._compat import string_types
import os
import re
import logging
import inspect
import datetime
import decimal


log = logging

def safe_import(path):
    module = path.split('.')
    g = __import__(module[0], fromlist=['*'])
    s = [module[0]]
    for i in module[1:]:
        mod = g
        if hasattr(mod, i):
            g = getattr(mod, i)
        else:
            s.append(i)
            g = __import__('.'.join(s), fromlist=['*'])
    return mod, g
        
def import_mod_attr(path):
    """
    Import string format module, e.g. 'uliweb.orm' or an object
    return module object and object
    """
    if isinstance(path, string_types):
        module, func = path.rsplit('.', 1)
        mod = __import__(module, fromlist=['*'])
        f = getattr(mod, func)
    else:
        f = path
        mod = inspect.getmodule(path)
    return mod, f

def import_attr(func):
    mod, f = import_mod_attr(func)
    return f

def myimport(module):
    mod = __import__(module, fromlist=['*'])
    return mod

class MyPkg(object):
    @staticmethod
    def resource_filename(module, path):
        mod = myimport(module)
        p = os.path.dirname(mod.__file__)
        if path:
            return os.path.join(p, path)
        else:
            return p
    
    @staticmethod
    def resource_listdir(module, path):
        d = MyPkg.resource_filename(module, path)
        return os.listdir(d)
    
    @staticmethod
    def resource_isdir(module, path):
        d = MyPkg.resource_filename(module, path)
        return os.path.isdir(d)

try:
    import pkg_resources as pkg
except:
    pkg = MyPkg

def extract_file(module, path, dist, verbose=False, replace=True):
    outf = os.path.join(dist, os.path.basename(path))
#    d = pkg.get_distribution(module)
#    if d.has_metadata('zip-safe'):
#        f = open(outf, 'wb')
#        f.write(pkg.resource_string(module, path))
#        f.close()
#        if verbose:
#            print 'Info : Extract %s/%s to %s' % (module, path, outf)
#    else:
    import shutil

    inf = pkg.resource_filename(module, path)
    sfile = os.path.basename(inf)
    if os.path.isdir(dist):
        dfile = os.path.join(dist, sfile)
    else:
        dfile = dist
    f = os.path.exists(dfile)
    if replace or not f:
        shutil.copy2(inf, dfile)
        if verbose:
            print('Copy %s to %s' % (inf, dfile))
  
def extract_dirs(mod, path, dst, verbose=False, exclude=None, exclude_ext=None, recursion=True, replace=True):
    """
    mod name
    path mod path
    dst output directory
    resursion True will extract all sub module of mod
    """
    default_exclude = ['.svn', '_svn', '.git']
    default_exclude_ext = ['.pyc', '.pyo', '.bak', '.tmp']
    exclude = exclude or []
    exclude_ext = exclude_ext or []
#    log = logging.getLogger('uliweb')
    if not os.path.exists(dst):
        os.makedirs(dst)
        if verbose:
            print('Make directory %s' % dst)
    for r in pkg.resource_listdir(mod, path):
        if r in exclude or r in default_exclude:
            continue
        fpath = os.path.join(path, r)
        if pkg.resource_isdir(mod, fpath):
            if recursion:
                extract_dirs(mod, fpath, os.path.join(dst, r), verbose, exclude, exclude_ext, recursion, replace)
        else:
            ext = os.path.splitext(fpath)[1]
            if ext in exclude_ext or ext in default_exclude_ext:
                continue
            extract_file(mod, fpath, dst, verbose, replace)

def match(f, patterns):
    from fnmatch import fnmatch
    
    flag = False
    for x in patterns:
        if fnmatch(f, x):
            return True
        
def walk_dirs(path, include=None, include_ext=None, exclude=None, 
        exclude_ext=None, recursion=True, file_only=False):
    """
    path directory path
    resursion True will extract all sub module of mod
    """
    default_exclude = ['.svn', '_svn', '.git']
    default_exclude_ext = ['.pyc', '.pyo', '.bak', '.tmp']
    exclude = exclude or []
    exclude_ext = exclude_ext or []
    include_ext = include_ext or []
    include = include or []

    if not os.path.exists(path):
        raise StopIteration
    
    for r in os.listdir(path):
        if match(r, exclude) or r in default_exclude:
            continue
        if include and r not in include:
            continue
        fpath = os.path.join(path, r)
        if os.path.isdir(fpath):
            if not file_only:
                yield os.path.normpath(fpath).replace('\\', '/')
            if recursion:
                for f in walk_dirs(fpath, include, include_ext, exclude, 
                    exclude_ext, recursion, file_only):
                    yield os.path.normpath(f).replace('\\', '/')
        else:
            ext = os.path.splitext(fpath)[1]
            if ext in exclude_ext or ext in default_exclude_ext:
                continue
            if include_ext and ext not in include_ext:
                continue
            yield os.path.normpath(fpath).replace('\\', '/')

def copy_dir(src, dst, verbose=False, check=False, processor=None):
    import shutil

#    log = logging.getLogger('uliweb')

    def _md5(filename):
        try:
            import hashlib
            a = hashlib.md5()
        except ImportError:
            import md5
            a = md5.new()
            
        a.update(open(filename, 'rb').read())
        return a.digest()
    
    if not os.path.exists(dst):
        os.makedirs(dst)

    if verbose:
        print("Processing %s" % src)
        
    for r in os.listdir(src):
        if r in ['.svn', '_svn', '.git']:
            continue
        fpath = os.path.join(src, r)
        
        if os.path.isdir(fpath):
            if os.path.abspath(fpath) != os.path.abspath(dst):
                copy_dir(fpath, os.path.join(dst, r), verbose, check, processor)
            else:
                continue
        else:
            ext = os.path.splitext(fpath)[1]
            if ext in ['.pyc', '.pyo', '.bak', '.tmp']:
                continue
            df = os.path.join(dst, r)
            if check:
                if os.path.exists(df):
                    a = _md5(fpath)
                    b = _md5(df)
                    if a != b:
                        print("Error: Target file %s is already existed, and "
                            "it not same as source one %s, so copy failed" % (fpath, dst))
                else:
                    if processor:
                        if processor(fpath, dst, df):
                            continue
                    shutil.copy2(fpath, dst)
                    if verbose:
                        print("Copy %s to %s" % (fpath, dst))
                    
            else:
                if processor:
                    if processor(fpath, dst, df):
                        continue
                shutil.copy2(fpath, dst)
                if verbose:
                    print("Copy %s to %s" % (fpath, dst))

def copy_dir_with_check(dirs, dst, verbose=False, check=True, processor=None):
#    log = logging.getLogger('uliweb')

    for d in dirs:
        if not os.path.exists(d):
            continue

        copy_dir(d, dst, verbose, check, processor)

ESCAPE = re.compile(r'[\x00-\x1f\\"\b\f\n\r\t]')
ESCAPE_DCT = {
    '\\': '\\\\',
    '"': '\\"',
    '\b': '\\b',
    '\f': '\\f',
    '\n': '\\n',
    '\r': '\\r',
    '\t': '\\t',
}
for i in range(0x20):
    ESCAPE_DCT.setdefault(chr(i), '\\u{0:04x}'.format(i))

def encode_basestring(s):
    """Return a JSON representation of a Python string

    """
    def replace(match):
        return ESCAPE_DCT[match.group(0)]
    return '"' + ESCAPE.sub(replace, s) + '"'

def encode_unicode(s):
    """Return a JSON representation of a Python unicode

    """
    return '"' + s.encode('unicode_escape') + '"'

def simple_value(v):
    from uliweb.i18n.lazystr import LazyString

    if callable(v):
        v = v()
    if isinstance(v, LazyString) or isinstance(v, decimal.Decimal) or isinstance(v, datetime.datetime):
        return str(v)
    else:
        return v

class JSONEncoder(object):
    def __init__(self, encoding='utf-8', unicode=False, default=None):
        self.encoding = encoding
        self.unicode = unicode
        self.default = default

    def iterencode(self, obj, key=False):
        if self.default:
            x = self.default(obj)
            obj = x
        if isinstance(obj, str):
            if self.unicode:
                yield encode_unicode(unicode(obj, self.encoding))
            else:
                yield encode_basestring(obj)
        elif isinstance(obj, unicode):
            if self.unicode:
                yield encode_unicode(obj)
            else:
                yield encode_basestring(obj.encode(self.encoding))
        elif obj is None:
            yield 'null'
        elif obj is True:
            yield 'true'
        elif obj is False:
            yield 'false'
        elif isinstance(obj, (int, long)):
            if key:
                yield '"' + str(obj) + '"'
            else:
                yield str(obj)
        elif isinstance(obj, float):
            if key:
                yield '"' + str(obj) + '"'
            else:
                yield str(obj)
        elif isinstance(obj, (list, tuple)):
            yield '['
            first = True
            for x in obj:
                if not first:
                    yield ','
                for y in self.iterencode(x):
                    yield y
                first = False
            yield ']'
        elif isinstance(obj, dict):
            yield '{'
            first = True
            for k, v in obj.iteritems():
                if not first:
                    yield ','
                for x in self.iterencode(k, key=True):
                    yield x
                yield ':'
                for y in self.iterencode(v):
                    yield y
                first = False
            yield '}'
        elif isinstance(obj, decimal.Decimal):
            yield str(obj)
        elif isinstance(obj, datetime.datetime):
            yield '"' + obj.strftime('%Y-%m-%d %H:%M:%S') + '"'
        elif isinstance(obj, datetime.date):
            yield '"' + obj.strftime('%Y-%m-%d') + '"'
        elif isinstance(obj, datetime.time):
            yield '"' + obj.strftime('%H:%M:%S') + '"'
        else:
            yield encode_basestring(str(obj))

    def encode(self, obj):
        return ''.join(self.iterencode(obj))

def json_dumps(obj, unicode=False, **kwargs):
    return JSONEncoder(unicode=unicode, default=simple_value, **kwargs).encode(obj)




"""
Compatible with py2 and py3, inspired by jinjin2, future, etc

common types & functions:

    name         2.x             3.x
    ------------ -----------     -----------
    unichr       unichr          chr
    unicode      unicode         str
    range        xrange          range
    string_types (str, unicode)  (str, )
    pickle       cPickle         pickle
    input        raw_input       input

StingIO, BytesIO

from io import StringIO, BytesIO

"""
import sys

PY3 = sys.version_info[0] == 3
PY2 = sys.version_info[0] == 2
PY26 = sys.version_info[0:2] == (2, 6)
PYPY = hasattr(sys, 'pypy_translation_info')
_identity = lambda x:x

if not PY2:
    unichr = chr
    range = range
    string_types = (str,)

    iterkeys = lambda d: iter(d.keys())
    itervalues = lambda d: iter(d.values())
    iteritems = lambda d: iter(d.items())

    import pickle

    def reraise(tp, value, tb=None):
        if value.__traceback__ is not tb:
            raise value.with_traceback(tb)
        raise value

    implements_iterator = _identity
    python_2_unicode_compatible = _identity

    ifilter = filter
    imap = map
    izip = zip

    def u(s, encoding='utf8'):
        if isinstance(s, str):
            return s
        else:
            return str(s)

    def b(s):
        if isinstance(s, bytes):
            return s
        else:
            if isinstance(s, str):
                return s.encode(encoding)
            else:
                return bytes(s)

    import builtins
    exec_ = getattr(builtins, "exec")

    get_next = lambda x: x.next

    input = input
    open = open

else:
    unichr = unichr
    range = xrange
    string_types = (str, unicode)

    iterkeys = lambda d: d.iterkeys()
    itervalues = lambda d: d.itervalues()
    iteritems = lambda d: d.iteritems()

    import cPickle as pickle
    from io import BytesIO, StringIO

    exec('def reraise(tp, value, tb=None):\n raise tp, value, tb')

    from itertools import imap, izip, ifilter

    def implements_iterator(cls):
        cls.next = cls.__next__
        del cls.__next__
        return cls

    def python_2_unicode_compatible(cls):
        cls.__unicode__ = cls.__str__
        cls.__str__ = lambda x: x.__unicode__().encode('utf-8')
        return cls

    def u(s, encoding='utf8'):
        if isinstance(s, unicode):
            return s
        else:
            return unicode(str(s), encoding)

    def b(s, encoding='utf8'):
        if isinstance(s, unicode):
            return s.decode(encoding)
        else:
            return str(s)

    def exec_(code, globs=None, locs=None):
        """Execute code in a namespace."""
        if globs is None:
            frame = sys._getframe(1)
            globs = frame.f_globals
            if locs is None:
                locs = frame.f_locals
            del frame
        elif locs is None:
            locs = globs
        exec("""exec code in globs, locs""")

    get_next = lambda x: x.__next__

    input = raw_input

    from io import open

try:
    next = next
except NameError:
    def next(it):
        return it.next()


def with_metaclass(meta, *bases):
    # This requires a bit of explanation: the basic idea is to make a
    # dummy metaclass for one level of class instanciation that replaces
    # itself with the actual metaclass.  Because of internal type checks
    # we also need to make sure that we downgrade the custom metaclass
    # for one level to something closer to type (that's why __call__ and
    # __init__ comes back from type etc.).
    #
    # This has the advantage over six.with_metaclass in that it does not
    # introduce dummy classes into the final MRO.
    class metaclass(meta):
        __call__ = type.__call__
        __init__ = type.__init__
        def __new__(cls, name, this_bases, d):
            if this_bases is None:
                return type.__new__(cls, name, (), d)
            return meta(name, bases, d)
    return metaclass('temporary_class', None, {})

modules_mapping = {
    'socketserver':'SocketServer',
    'queue':'Queue',
    'configparser':'ConfigParser',

    'html.entities':'htmlentitydefs',
    'html.parser':'HTMLParser',

    'http.client':'httplib',
    'http.server':['BaseHTTPServer', 'CGIHTTPServer', 'SimpleHTTPServer', 'CGIHTTPServer'],

        # from BaseHTTPServer
        # from CGIHTTPServer
        # from SimpleHTTPServer
        # from CGIHTTPServer

    'http.cookies':'Cookie',
    'http.cookiejar':'cookielib',

    'urllib.parse':['urlparse', 'urllib'],

        # from urlparse import (ParseResult, SplitResult, parse_qs, parse_qsl,
        #           urldefrag, urljoin, urlparse, urlsplit,
        #               urlunparse, urlunsplit)
        # from urllib import (quote,
        #             quote_plus,
        #             unquote,
        #             unquote_plus,
        #             urlencode,
        #             splitquery)

    'urllib.request':['urllib', 'urllib2', 'urlparse'],

        # from urllib import (pathname2url,
        #                     url2pathname,
        #                     getproxies,
        #                     urlretrieve,
        #                     urlcleanup,
        #                     URLopener,
        #                     FancyURLopener,
        #                     proxy_bypass)

        # from urllib2 import (
        #                  AbstractBasicAuthHandler,
        #                  AbstractDigestAuthHandler,
        #                  BaseHandler,
        #                  CacheFTPHandler,
        #                  FileHandler,
        #                  FTPHandler,
        #                  HTTPBasicAuthHandler,
        #                  HTTPCookieProcessor,
        #                  HTTPDefaultErrorHandler,
        #                  HTTPDigestAuthHandler,
        #                  HTTPErrorProcessor,
        #                  HTTPHandler,
        #                  HTTPPasswordMgr,
        #                  HTTPPasswordMgrWithDefaultRealm,
        #                  HTTPRedirectHandler,
        #                  HTTPSHandler,
        #                  URLError,
        #                  build_opener,
        #                  install_opener,
        #                  OpenerDirector,
        #                  ProxyBasicAuthHandler,
        #                  ProxyDigestAuthHandler,
        #                  ProxyHandler,
        #                  Request,
        #                  UnknownHandler,
        #                  urlopen,
        #                 )

        # from urlparse import (
        #                  urldefrag
        #                  urljoin,
        #                  urlparse,
        #                  urlunparse,
        #                  urlsplit,
        #                  urlunsplit,
        #                  parse_qs,
        #                  parse_q"
        #                 )

    'urllib.error':['urllib', 'urllib2'],

        # from urllib import ContentTooShortError
        # from urllib2 import URLError, HTTPError

    'xmlrpc.client':'xmlrpclib',
    'xmlrpc.server':'xmlrpclib',
}
def import_(module, objects=None, via=None):
    """
    :param module: py3 compatiable module path
    :param objects: objects want to imported, it should be a list
    :param via: for some py2 module, you should give the import path according the
        objects which you want to imported
    :return: object or module
    """
    if PY3:
        mod = __import__(module, fromlist=['*'])
    else:
        path = modules_mapping.get(module)
        if not path:
            raise Exception("Can't find the module %s in mappings." % module)
        if isinstance(path, list):
            if not via:
                raise Exception("You should give a via parameter to enable import from py2.")
            path = via
        mod = __import__(path, fromlist=['*'])

    if objects:
        if not isinstance(objects, (list, tuple)):
            raise Exception("objects parameter should be a list or tuple.")
        r = []
        for x in objects:
            r.append(getattr(mod, x))
        if len(r) > 1:
            return tuple(r)
        else:
            return r[0]
    else:
        return mod



from __future__ import print_function, absolute_import, unicode_literals
from ._compat import exec_, u, string_types, open

import re
import os
import io
import cgi

__templates_temp_dir__ = 'tmp/templates_temp'
__options__ = {'use_temp_dir':False}
__nodes__ = {}   #user defined nodes

BEGIN_TAG = '{{'
END_TAG = '}}'

class TemplateException(Exception): pass
class ContextPopException(TemplateException):
    "pop() has been called more times than push()"
    pass

def use_tempdir(dir=''):
    global __options__, __templates_temp_dir__
    
    if dir:
        __templates_temp_dir__ = dir
        __options__['use_temp_dir'] = True
        if not os.path.exists(__templates_temp_dir__):
            os.makedirs(__templates_temp_dir__)

def set_options(**options):
    """
    default use_temp_dir=False
    """
    __options__.update(options)

def get_temp_template(filename):
    if __options__['use_temp_dir']:
        f, filename = os.path.splitdrive(filename)
        filename = filename.replace('\\', '_')
        filename = filename.replace('/', '_')
        f, ext = os.path.splitext(filename)
        filename = f + '.py'
        return os.path.normcase(os.path.join(__templates_temp_dir__, filename))
    return filename

def register_node(name, node):
    assert issubclass(node, Node)
    __nodes__[name] = node

def reindent(text):
    lines=text.split('\n')
    new_lines=[]
    credit=k=0
    for raw_line in lines:
        line=raw_line.strip()
        if not line or line[0]=='#':
            new_lines.append(line)
            continue
        if line[:5]=='elif ' or line[:5]=='else:' or    \
            line[:7]=='except:' or line[:7]=='except ' or \
            line[:7]=='finally:' or line[:5]=='with ':
                k=k+credit-1
        if k<0: k=0
        new_lines.append('    '*k+line)
        credit=0
        if line=='pass' or line[:5]=='pass ':
            credit=0
            k-=1
#        if line=='return' or line[:7]=='return ' or \
#            line=='continue' or line[:9]=='continue ' or \
#            line=='break' or line[:6]=='break':
#            credit=1
#            k-=1
        if line[-1:]==':' or line[:3]=='if ':
            k+=1
    text='\n'.join(new_lines)
    return text

def get_templatefile(filename, dirs, default_template=None, skip='', skip_original=''):
    """
    Fetch the template filename according dirs
    :para skip: if the searched filename equals skip, then using the one before.
    """
    def _file(filename, dirs):
        for d in dirs:
            _f = os.path.normcase(os.path.join(d, filename))
            if os.path.exists(_f):
                yield _f
        raise StopIteration
    
    filename = os.path.normcase(filename)
    skip = os.path.normcase(skip)
    skip_original = os.path.normcase(skip_original)
    
    if os.path.exists(filename):
        return filename
    
    if filename and dirs:
        _files = _file(filename, dirs)
        if skip_original == filename:
            for f in _files:
                if f == skip:
                    break
                
        for f in _files:
            if f != skip:
                return f
            else:
                continue
            
    if default_template:
        if isinstance(default_template, (list, tuple)):
            for i in default_template:
                f = get_templatefile(i, dirs)
                if f:
                    return f
        else:
            return get_templatefile(default_template, dirs)

def parse_arguments(text, key='with'):
    r = re.compile(r'\s+%s\s+' % key)
    k = re.compile(r'^\s*([\w][a-zA-Z_0-9]*\s*)=\s*(.*)')
    b = r.split(text)
    if len(b) == 1:
        name, args, kwargs = b[0], (), {}
    else:
        name = b[0]
        s = b[1].split(',')
        args = []
        kwargs = {}
        for x in s:
            ret = k.search(x)
            if ret:
                kwargs[ret.group(1)] = ret.group(2)
            else:
                args.append(x)
                
    return name, args, kwargs

def eval_vars(vs, vars, env):
    if isinstance(vs, (tuple, list)):
        return [eval_vars(x, vars, env) for x in vs]
    elif isinstance(vs, dict):
        return dict([(x, eval_vars(y, vars, env)) for x, y in vs.items()])
    else:
        return eval(vs, env, vars)

def get_tag(begin_tag, end_tag):
    r = (
        '(' + re.escape(begin_tag) + '##.*?##' + re.escape(end_tag) + '|' +
            re.escape(begin_tag) + '.*?' + re.escape(end_tag) + 
        ')')
    return re.compile(r, re.DOTALL|re.M)

r_tag = re.compile('^#uliweb-template-tag:(.+?),(.+?)(:\r|\n|\r\n)')

class Node(object):
    block = 0
    var = False
    def __init__(self, value=None, content=None, template=None):
        self.value = value
        self.content = content
        self.template = template
        
    def __str__(self):
        if self.value:
            return self.value
        else:
            return ''
        
    def __repr__(self):
        return self.__str__()
    
class BaseBlockNode(Node):
    def __init__(self, name='', content=None):
        self.nodes = []
        self.name = name
        self.content = content
        self.block = 1
        
    def add(self, node):
        self.nodes.append(node)

    def end(self):
        pass
    
    def __repr__(self):
        s = ['{{BaseBlockNode %s}}' % self.name]
        for x in self.nodes:
            s.append(repr(x))
        s.append('{{end}}')
        return ''.join(s)
    
    def __str__(self):
        return self.render()
    
    def render(self):
        s = []
        for x in self.nodes:
            s.append(u(str(x)))
        return ''.join(s)
    
class BlockNode(BaseBlockNode):
    def add(self, node):
        self.nodes.append(node)
        if isinstance(node, BlockNode):
            v = self.content.root.block_vars.setdefault(node.name, [])
            v.append(node)
        
    def merge(self, content):
        self.nodes.extend(content.nodes)
    
    def render(self, top=True):
        """
        Top: if output the toppest block node
        """
        if top and self.name in self.content.root.block_vars and self is not self.content.root.block_vars[self.name][-1]:
            return self.content.root.block_vars[self.name][-1].render(False)
        
        s = []
        for x in self.nodes:
            if isinstance(x, BlockNode):
                if x.name in self.content.root.block_vars:
                    s.append(u((self.content.root.block_vars[x.name][-1])))
                else:
                    s.append(u(str(x)))
            else:
                s.append(u(str(x)))
        return ''.join(s)
        
class SuperNode(Node):
    def __init__(self, parent, content):
        self.parent = parent
        self.content = content
        
    def __str__(self):
        for i, v in enumerate(self.content.root.block_vars[self.parent.name]):
            if self.parent is v:
                if i > 0:
                    return self.content.root.block_vars[self.parent.name][i-1].render(False)
        return ''
    
    def __repr__(self):
        return '{{super}}'

class Content(BaseBlockNode):
    def __init__(self, root=None):
        self.nodes = []
        self.block_vars = {}
        self.begin = []
        self.end = []
        self.root = root or self
        
    def add(self, node):
        self.nodes.append(node)
        if isinstance(node, BlockNode):
            if node.name:
                v = self.block_vars.setdefault(node.name, [])
                v.append(node)
        
    def merge(self, content):
        self.nodes.extend(content.nodes)
        for k, v in content.block_vars.items():
            d = self.block_vars.setdefault(k, [])
            d.extend(v)
        content.root = self.root
        
    def clear_content(self):
        self.nodes = []
    
    def __str__(self):
        s = self.begin[:]
        for x in self.nodes:
            s.append(u(str(x)))
        s.extend(self.end)
        return ''.join(s)
    
    def __repr__(self):
        s = []
        for x in self.nodes:
            s.append(repr(x))
        return ''.join(s)

class Context(object):
    "A stack container for variable context"
    def __init__(self, dict_=None):
        dict_ = dict_ or {}
        self.dicts = [dict_]
        self.dirty = True
        self.result = None

    def __repr__(self):
        return repr(self.dicts)

    def __iter__(self):
        for d in self.dicts:
            yield d

    def push(self):
        d = {}
        self.dicts = [d] + self.dicts
        self.dirty = True
        return d

    def pop(self):
        if len(self.dicts) == 1:
            raise ContextPopException
        return self.dicts.pop(0)
        self.dirty = True

    def __setitem__(self, key, value):
        "Set a variable in the current context"
        self.dicts[0][key] = value
        self.dirty = True

    def __getitem__(self, key):
        "Get a variable's value, starting at the current context and going upward"
        for d in self.dicts:
            if key in d:
                return d[key]
        raise KeyError(key)

    def __delitem__(self, key):
        "Delete a variable from the current context"
        del self.dicts[0][key]
        self.dirty = True

    def has_key(self, key):
        for d in self.dicts:
            if key in d:
                return True
        return False

    __contains__ = has_key

    def get(self, key, otherwise=None):
        for d in self.dicts:
            if key in d:
                return d[key]
        return otherwise

    def update(self, other_dict):
        "Like dict.update(). Pushes an entire dictionary's keys and values onto the context."
        if not hasattr(other_dict, '__getitem__'): 
            raise TypeError('other_dict must be a mapping (dictionary-like) object.')
        self.dicts[0].update(other_dict)
#        self.dicts = [other_dict] + self.dicts
        self.dirty = True
        return other_dict
    
    def to_dict(self):
        if not self.dirty:
            return self.result
        else:
            d = {}
            for i in reversed(self.dicts):
                d.update(i)
            self.result = d
            self.dirty = False
        return d
        
__nodes__['block'] = BlockNode

class Out(object):
    encoding = 'utf-8'
    
    def __init__(self):
        self.buf = io.StringIO()
        
    def _str(self, text):
        return u(text)

    def write(self, text, escape=True):
        s = self._str(text)
        if escape:
            self.buf.write(cgi.escape(s))
        else:
            self.buf.write(s)
            
    def xml(self, text):
        self.write(self._str(text), escape=False)
        
#    def json(self, text):
#        from datawrap import dumps
#        self.write(dumps(text))
#
    def getvalue(self):
        return self.buf.getvalue()

class Template(object):
    def __init__(self, text='', vars=None, env=None, dirs=None, 
        default_template=None, use_temp=False, compile=None, skip_error=False, 
        encoding='utf-8', begin_tag=None, end_tag=None):
        self.text = u(text)
        self.filename = None
        self.vars = vars or {}
        if not isinstance(env, Context):
            env = Context(env)
        self.env = env
        self.dirs = dirs or '.'
        self.default_template = default_template
        self.use_temp = use_temp
        self.compile = compile
        self.writer = 'out.write'
        self.content = Content()
        self.stack = [self.content]
        self.depend_files = []  #used for template dump file check
        self.callbacks = []
        self.exec_env = {}
        self.root = self
        self.skip_error = skip_error
        self.encoding = encoding
        self.begin_tag = begin_tag or BEGIN_TAG
        self.end_tag = end_tag or END_TAG
        
        for k, v in __nodes__.items():
            if hasattr(v, 'init'):
                v.init(self)
        
    def add_callback(self, callback):
        if not callback in self.root.callbacks:
            self.root.callbacks.append(callback)
            
    def add_exec_env(self, name, obj):
        self.root.exec_env[name] = obj
        
    def add_root(self, root):
        self.root = root
        
    def set_filename(self, filename):
        fname = get_templatefile(filename, self.dirs, self.default_template)
        if not fname:
            raise TemplateException("Can't find the template %s" % filename)
        self.filename = fname
        self.original_filename = filename
    
    def _get_parameters(self, value):
        def _f(*args, **kwargs):
            return args, kwargs
        d = self.env.to_dict()
        d['_f'] = _f
        try:
            args, kwargs = eval("_f(%s)" % value, d, self.vars)
        except:
            if self.skip_error:
                return (None,), {}
            else:
                raise
        return args, kwargs
    
    def parse(self):
        text = self.text
        extend = None  #if need to process extend node
        for i in get_tag(self.begin_tag, self.end_tag).split(text):
            if i:
                if len(self.stack) == 0:
                    raise TemplateException("The 'end' tag is unmatched, please check if you have more '{{end}}'")
                top = self.stack[-1]
                #process multiline comment
                if i.startswith(self.begin_tag+'##'):
                    continue
                in_tag = i.startswith(self.begin_tag)
                if in_tag:
                    line = i[2:-2].strip()
                    if not line:
                        continue
                    elif line.startswith('='):
                        name, value = '=', line[1:].strip()
                    elif line.startswith('<<'):
                        name, value = '<<', line[2:].strip()
                    else:
                        v = line.split(' ', 1)
                        if len(v) == 1:
                            name, value = v[0], ''
                        else:
                            name, value = v
                    if name in __nodes__:
                        node_cls = __nodes__[name]
                        #this will pass top template instance and top content instance to node_cls
                        node = node_cls(value.strip(), self.content)
                        if node.block:
                            top.add(node)
                            self.stack.append(node)
                        else:
                            buf = u(str(node))
                            if buf:
                                top.add(buf)
                    elif name == 'super':
                        t = self.stack[-1]
                        if isinstance(t, BaseBlockNode):
                            node = SuperNode(t, self.content)
                            top.add(node)
                    elif name == 'end':
                        #add block.end process
                        #if end() return something, it'll be append to top node
                        t = self.stack.pop()
                        top = self.stack[-1]
                        if t.block and hasattr(t, 'end'):
                            buf = t.end()
                            if buf:
                                top.add(buf)
                    elif name == '=':
                        buf = "%s(%s)\n" % (self.writer, value)
                        top.add(buf)
                    elif name == 'BEGIN_TAG':
                        buf = "%s('{{')\n" % self.writer
                        top.add(buf)
                    elif name == 'END_TAG':
                        buf = "%s('}}')\n" % self.writer
                        top.add(buf)
                    elif name == '<<':
                        buf = "%s(%s, escape=False)\n" % (self.writer, value)
                        top.add(buf)
#                    elif name == 'T=':
#                        if not self._parse_template(top, value):
#                            buf = "%s(%s)\n" % (self.writer, value)
#                            top.add(buf)
#                    elif name == 'T<<':
#                        if not self._parse_template(top, value):
#                            buf = "%s(%s, escape=False)\n" % (self.writer, value)
#                            top.add(buf)
                    elif name == 'include':
                        self._parse_include(top, value)
                    elif name == 'embed':
                        self._parse_text(top, value)
                    elif name == 'extend':
                        extend = value
                    else:
                        if line and in_tag:
                            top.add(line+'\n')
                else:
                    buf = "%s(%r, escape=False)\n" % (self.writer, i)
                    top.add(buf)
                    
        if extend:
            self._parse_extend(extend)
        # if self.encoding:
        #     pre = '#coding=%s\n' % self.encoding
        # else:
        #     pre = ''
#        return reindent(pre + str(self.content))
        return reindent(u(str(self.content)))
    
    def _parse_template(self, content, var):
        if var in self.vars:
            v = self.vars[var]
        else:
            return False
        
        #add v.__template__ support
        if hasattr(v, '__template__'):
            text = u(str(v.__template__(var)))
            if text:
                t = Template(text, self.vars, self.env, self.dirs)
                t.parse()
                t.add_root(self)
                content.merge(t.content)
                return True
        
        return False

    def _parse_text(self, content, var):
        try:
            text = u(str(eval(var, self.env.to_dict(), self.vars)))
        except:
            if self.skip_error:
                text = ''
            else:
                raise
        if text:
            t = Template(text, self.vars, self.env, self.dirs)
            t.parse()
            t.add_root(self)
            content.merge(t.content)
    
    def _parse_include(self, content, value):
        self.env.push()
        try:
            args, kwargs = self._get_parameters(value)
            filename = args[0]
            self.env.update(kwargs)
            fname = get_templatefile(filename, self.dirs, skip=self.filename, skip_original=self.original_filename)
            if not fname:
                raise TemplateException("Can't find the template %s" % filename)
            
            self.depend_files.append(fname)
            
            f = open(fname, 'r')
            text, begin_tag, end_tag = self.get_text(f.read(), inherit_tags=False)
            f.close()
            t = Template(text, self.vars, self.env, self.dirs, begin_tag=begin_tag, end_tag=end_tag)
            t.set_filename(fname)
            t.add_root(self)
            t.parse()
            content.merge(t.content)
        finally:
            self.env.pop()
        
    def _parse_extend(self, value):
        """
        If the extend template is the same name with current file, so it
        means that it should use parent template
        """
        self.env.push()
        try:
            args, kwargs = self._get_parameters(value)
            filename = args[0]
            self.env.update(kwargs)
            fname = get_templatefile(filename, self.dirs, skip=self.filename, skip_original=self.original_filename)
            if not fname:
                raise TemplateException("Can't find the template %s" % filename)
            
            self.depend_files.append(fname)
            
            f = open(fname, 'r')
            text, begin_tag, end_tag = self.get_text(f.read(), inherit_tags=False)
            f.close()
            t = Template(text, self.vars, self.env, self.dirs, begin_tag=begin_tag, end_tag=end_tag)
            t.set_filename(fname)
            t.add_root(self)
            t.parse()
            self.content.clear_content()
            t.content.merge(self.content)
            self.content = t.content
        finally:
            self.env.pop()
            
    def get_parsed_code(self):
        if self.use_temp:
            f = get_temp_template(self.filename)
            if os.path.exists(f):
                fin = open(f, 'r')
                modified = False
                files = [self.filename]
                line = fin.readline()
                if line.startswith('#uliweb-template-files:'):
                    files.extend(line[1:].split())
                else:
                    fin.seek(0)
                
                for x in files:
                    if os.path.getmtime(x) > os.path.getmtime(f):
                        modified = True
                        break
                    
                if not modified:
                    text = fin.read()
                    fin.close()
                    return True, f, text
        
        if self.filename and not self.text:
            self.text, self.begin_tag, self.end_tag = self.get_text(open(self.filename, 'r').read())
        return False, self.filename, self.parse()
        
    def get_text(self, text, inherit_tags=True):
        """
        Detect template tag definition in the text
        If inherit_tags is True, then use current begin and end tag string, 
        or use default tag string
        """
        b = r_tag.search(text)
        if b:
            begin_tag, end_tag = b.group(1), b.group(2)
            text = text[b.end():]
        else:
            if inherit_tags:
                begin_tag = self.begin_tag
                end_tag = self.end_tag
            else:
                begin_tag = BEGIN_TAG
                end_tag = END_TAG
        return text, begin_tag, end_tag
    
    def __call__(self):
        use_temp_flag, filename, code = self.get_parsed_code()
        
        if not use_temp_flag and self.use_temp:
            f = get_temp_template(filename)
            try:
                fo = open(f, 'w')
                fo.write('#uliweb-template-files:%s\n' % ' '.join(self.depend_files))
                fo.write(code)
                fo.close()
            except:
                pass
        
        return self._run(code, filename or 'template')
        
    def _run(self, code, filename):
        def f(_vars, _env):
            def defined(v, default=None):
                _v = default
                if v in _vars:
                    _v = _vars[v]
                elif v in _env:
                    _v = _env[v]
                return _v
            return defined

        e = {}
        if isinstance(self.env, Context):
            new_e = self.env.to_dict()
        else:
            new_e = self.env
        e.update(new_e)
        e.update(self.vars)
        out = Out()
        e['out'] = out
        e['Out'] = Out
        e['xml'] = out.xml
        e['_vars'] = self.vars
        e['defined'] = f(self.vars, self.env)
        e['_env'] = e
        
        e.update(self.exec_env)
        
        if isinstance(code, string_types):
            if self.compile:
                code = self.compile(code, filename, 'exec', e)
            else:
                code = compile(code, filename, 'exec')
        exec_(code, e)
        text = out.getvalue()
        
        for f in self.callbacks:
            text = f(text, self, self.vars, e)

        return text
    
def template_file(filename, vars=None, env=None, dirs=None, default_template=None, compile=None, **kwargs):
    t = Template('', vars, env, dirs, default_template, use_temp=__options__['use_temp_dir'], compile=compile, **kwargs)
    t.set_filename(filename)
    return t()

def template(text, vars=None, env=None, dirs=None, default_template=None, **kwargs):
    t = Template(text, vars, env, dirs, default_template, **kwargs)
    return t()



#! /usr/bin/env python
#coding=utf-8
#
# Author: limodou@gmail.com
# This program is based on pyPEG
#
# license: BSD
#
from __future__ import print_function, absolute_import, unicode_literals
from ._compat import open
import os, sys
import re
import traceback

sys.path.insert(0, '.')
from par.md import parseHtml
from .commands import call, register_command, Command, get_answer, get_input
from optparse import make_option
from . import template
from .utils import log
import shutil
from copy import deepcopy

__author__ = 'limodou'
__author_email__ = 'limodou@gmail.com'
__url__ = 'https://github.com/limodou/parm'
__license__ = 'BSD'
__version__ = '1.11'

#import parm project config module
try:
    import conf
    if not hasattr(conf, 'encoding'):
        conf.encoding = 'utf8'
except:
    conf = None

def fix_dir(path):
    """
    Check if directory is existed, if not then create it
    """
    _dir = os.path.dirname(path)
    if not os.path.exists(_dir):
        os.makedirs(_dir)
        
class InitCommand(Command):
    name = 'init'
    help = "Init a parm project environment. It'll create a config.py file."
    
    def handle(self, options, global_options, *args):
        from .utils import extract_dirs, pkg, extract_file
        from shutil import copy
        
        if os.path.exists('conf.py'):
            has_conf = True
        else:
            has_conf = False
            
        d = {}
        
        d['project'] = getattr('conf', 'project', 'Parm')
        d['copyright'] = getattr('conf', 'copyright', '2014, Limodou')
        d['version'] = getattr('conf', 'version', __version__)
        d['theme'] = getattr(conf, 'theme', 'bootstrap')
        d['theme'] = 'bootstrap'
        d['template_dirs'] = getattr(conf, 'template_dirs', 'templates')
        d['disqus'] = getattr(conf, 'disqus', '')
        d['domain'] = getattr(conf, 'domain', '')
        d['encoding'] = getattr(conf, 'encoding', 'utf8')
        
        if has_conf:
            create = get_answer("Create config file", quit='q') == 'Y'
        
        if not has_conf or (has_conf and create):
            d['project'] = get_input("Project name [%s]:"%d['project'], default=d['project'])
            d['copyright'] = get_input("Copyright information [%s]:" % d['copyright'], default=d['copyright'])
            d['version'] = get_input("Version [%s]:" % d['version'], default=d['version'])
            d['theme'] = get_input("Choice theme (bootstrap, semantic) [%s]:" % d['theme'], choices=['bootstrap', 'semantic'], default=d['theme'])
            d['disqus'] = get_input("Disqus account name [%s]:" % d['disqus'], default=d['disqus'])
            d['search'] = get_answer("If you want to add search input") == 'Y'
            d['domain'] = get_input("Domain name used for search [%s]:" % d['domain'], default=d['domain'])
            d['encoding'] = get_input("Encoding of the MarkDown files [%s]:" % d['encoding'], default=d['encoding'])

            if d['theme'] == 'bootstrap':
                d['tag_class'] = """
'table':'table table-bordered',
'pre':'+prettyprint',
"""
            elif d['theme'] == 'semantic':
                d['tag_class'] = """
'table':'ui collapsing celled table segment',
'pre':'+prettyprint',
"""
            
            conf_file = pkg.resource_filename('parm', 'templates/env/conf.py.txt')
            text = template.template_file(conf_file, d).replace('\r\n', '\n')
            f = open('conf.py', 'w')
            f.write(text)
            f.close()
        
        run = False
        if os.path.exists(d['template_dirs']):
            print("Template directory [%s] is already existed! If you've changed them, please deal with manually, otherwise the content will be overwritten." % d['template_dirs'])
            if get_answer("Overwrite template files") == 'Y':
                run = True
        else:
            run = True
        
        if run:
            print('Copy %s to ./%s' % ('theme/%s/templates' % d['theme'], d['template_dirs']))
            extract_dirs('parm', 'templates/theme/%s/templates' % d['theme'], 
                d['template_dirs'])
                
        if get_answer("Copy init files [index.md*]") == 'Y':
            for f in ['index.md', 'introduction.md', 'exclude.txt']:
                if os.path.exists(f):
                    print('%s is already existed, so just skip it' % f)
                else:
                    print('Copy templates/env/%s to ./%s' % (f, f))
                    extract_file('parm', 'templates/env/%s' % f, '.')
        
register_command(InitCommand)

from par.md import MarkdownHtmlVisitor

class MDVisitor(MarkdownHtmlVisitor):
    def visit_check_radio(self, node):
        tag = []
        _append = tag.append
        _start = node.text.startswith
        if _start('[*') or _start('[X') or _start('[x'):
            _append('<i class="fa fa-check-square-o"')
        elif _start('['):
            _append('<i class="fa fa-square-o"')
        elif _start('<*') or _start('<X') or _start('<x'):
            _append('<i class="fa fa-dot-circle-o"')
        else:
            _append('<i class="circle-o"')
        _append(' style="color:gray;"')
        _append('\></i> ')
        return ''.join(tag)

class MakeCommand(Command):
    name = 'make'
    help = "Make parm project. It'll create all markdown files to html files."
    has_options = True
    option_list = (
        make_option('-d', dest='directory', default='html',
            help='Output directory of converted files.'),
        make_option('-c', dest='copy', action='store_true', default=False,
                    help='Copy markdown to destination directory.'),
    )
    
    r_header = re.compile(r'<h(\d)(.*?)>(.*?)(<a.*?/a>)\s*</h\1>', re.IGNORECASE)
    r_tag = re.compile(r'<.*?>')
    r_id = re.compile(r'id="([^"]*)"')
    
    def handle(self, options, global_options, *args):
        from .utils import extract_dirs, copy_dir, walk_dirs, import_attr, json_dumps
        from .md_ext import new_code_comment, toc, include, code
        from .mermaid_ext import graph
        from functools import partial
        from shutil import copy2

        if not conf:
            log.error('Current directory is not a parm project')
            sys.exit(1)
            
        #make output directory
        if not os.path.exists(options.directory):
            print('Make directories [%s]' % options.directory)
            os.makedirs(options.directory)
            
        #get theme config
        theme = getattr(conf, 'theme', 'semantic')
        print('Using theme [%s]' % theme)
        
        #copy static files
        print('Copy %s to %s' % ('theme/%s/static' % theme, os.path.join(options.directory, 'static')))
        extract_dirs('parm', 'templates/theme/%s/static' % theme, 
            os.path.join(options.directory, 'static'))
            
        dst_dir = os.path.normpath(os.path.abspath(options.directory)).replace('\\', '/') + '/'
        
        #process exclude
        if os.path.exists('exclude.txt'):
            _exclude = open('exclude.txt').read().splitlines()
        else:
            _exclude = []
        _exclude.extend([options.directory, conf.template_dirs])
        files = list(walk_dirs('.', exclude=_exclude))
        headers = {}
        relations = {}
        
        #prepare block process handlers
        blocks = {}
        blocks['code-comment'] = new_code_comment
        blocks['toc'] = partial(toc, headers=headers, relations=relations)
        blocks['include'] = include
        blocks['graph'] = graph
        blocks['include-code'] = code

        #according theme import different blocks
        mod_path = 'par.%s_ext.blocks' % theme
        b = import_attr(mod_path)
        blocks.update(b)
        
        output_files = {}
        template_dirs = conf.template_dirs
        
        while 1:
            if not files:
                break
            
            path = files.pop(0)
            fname, ext = os.path.splitext(path)
            if os.path.isfile(path) and (ext in conf.source_suffix):
                if fname == conf.master_doc and len(files)>0:
                    files.append(path)
                    continue
                
                with open(path, encoding=conf.encoding) as f:
                    
                    data = {}
                    data['conf'] = conf
                    data['json_dumps'] = json_dumps
                    
                    #process markdown convert
                    try:
                        data['body'] = parseHtml(f.read(),
                            '',
                            conf.tag_class,
                            block_callback=blocks,
                            filename=path,
                            visitor=MDVisitor)
                    except Exception as e:
                        traceback.print_exc()
                        type, value, tb = sys.exc_info()
                        txt = ''.join(traceback.format_exception(type, value, tb))

                        data['body'] = txt
                        print ('Error: convert {} error'.format(path))
                    page_nav = relations.get(fname, {})
                    data['prev'] = page_nav.get('prev', {})
                    data['next'] = page_nav.get('next', {})
                    data['relpath'] = self.fix_relpath(path)
                    data['source'] = '<a href="%s/%s">%s</a>' % (data['relpath'], path, conf.download_source)
                    
                    #parse header from text
                    h = headers.setdefault(path, [])
                    title = self.parse_headers(path, data['body'], h)
                    data['subject'] = title
                    if title:
                        data['title'] = title + ' - ' + conf.project
                    else:
                        if fname != conf.master_doc:
                            print('Error: Heading 1 not found in file %s' % path)
                            continue
                        else:
                            data['title'] = conf.project

                    #convert conf attributes to data
                    for k in dir(conf):
                        if not k.startswith('_'):
                            data[k] = getattr(conf, k)
                       
                    #process template
                    template_file = conf.templates.get(fname, conf.templates.get('*', 'default.html'))
                    hfilename = os.path.join(options.directory, fname + '.html').replace('\\', '/')
                    fix_dir(hfilename)
                    with open(hfilename, 'w', encoding=conf.encoding) as fh:
                        print('Convert %s to %s' % (path, hfilename))
                        fh.write(template.template_file(template_file, data, dirs=[template_dirs]))
                        #if not copy, md file will not be copied
                        if options.copy:
                            copy2(path, os.path.join(options.directory, path))
                    output_files[fname] = hfilename
                    
            else:
                if os.path.isdir(path):
                    dpath = os.path.join(dst_dir, path)
                    if not os.path.exists(dpath):
                        print('Makedir %s' % os.path.join(dst_dir, path))
                        os.makedirs(dpath)
                else:
                    print('Copy %s to %s' % (path, os.path.join(dst_dir, path)))
                    shutil.copy(path, os.path.join(dst_dir, path))
                
        prev_next_template_top = """{{if prev:}}<div class="chapter-prev chapter-top pull-left">
    <a prev-chapter href="{{<< prev['link']}}"><i class="fa fa-long-arrow-left"></i> {{=prev['title']}}</a>
</div>{{pass}}
{{if next:}}<div class="chapter-next chapter-top pull-right">
    <a next-chapter href="{{<< next['link']}}">{{=next['title']}} <i class="fa fa-long-arrow-right"></i></a>
</div>{{pass}}"""
        prev_next_template_down = """{{if prev:}}<div class="chapter-prev chapter-down pull-left">
    <a prev-chapter href="{{<< prev['link']}}"><i class="fa fa-long-arrow-left"></i> {{=prev['title']}}</a>
</div>{{pass}}
{{if next:}}<div class="chapter-next chapter-down pull-right">
    <a next-chapter href="{{<< next['link']}}">{{=next['title']}} <i class="fa fa-long-arrow-right"></i></a>
</div>{{pass}}"""
        
        for name, f in output_files.items():
            text = open(f, encoding=conf.encoding).read()
            
            x = relations.get(name, {})
            data = {}
            data['prev'] = x.get('prev', {})
            if data['prev']:
                relpath = self.fix_relpath(data['prev']['link'])
                data['prev']['link'] = relpath + '/' + data['prev']['link']
            data['next'] = x.get('next', {})
            if data['next']:
                relpath = self.fix_relpath(data['next']['link'])
                data['next']['link'] = relpath + '/' + data['next']['link']
            
            prev_next_text_top = template.template(prev_next_template_top, data)
            prev_next_text_down = template.template(prev_next_template_down, data)
            text = text.replace('<!-- prev_next_top -->', prev_next_text_top)
            text = text.replace('<!-- prev_next_down -->', prev_next_text_down)
            with open(f, 'w', encoding=conf.encoding) as fh:
                fh.write(text)

    def fix_relpath(self, path):
        c = path.count('/')
        if c == 0:
            relpath = '.'
        elif c == 1:
            relpath = '..'
        else:
            relpath = '..' + '/..' * (c - 1)
        return relpath

    def parse_headers(self, filename, text, headers):
        title = None
        for x in MakeCommand.r_header.findall(text):
            fname, ext = os.path.splitext(filename)
            d = {
                'level':int(x[0]), 
                'title':MakeCommand.r_tag.sub('', x[2]),
                'link':'%s.html' % fname,
            }
            _id = MakeCommand.r_id.search(x[1])
            if _id:
                d['id'] = _id.group(1)
            else:
                log.error("Can't find anchor for head line %s:%s" % (filename, d['title']))
                d['id'] = None
                
            headers.append(d)
            if d['level'] == 1:
                title = d['title']
        return title
            
register_command(MakeCommand)

class Rst2MdCommand(Command):
    name = 'rst2md'
    help = ("Convert reStructuredText to Markdown. \n\nThis tool can't convert "
        "everything of rst to markdown very well, so you need to modify the output "
        "files manually, but it's a good start point")
    args = '[rstfile, rstfile...]'
    option_list = (
        make_option('-e', dest='extension', default='.rst',
            help='Extension of the reStructuredText file, default is ".rst".'),
        make_option('-d', dest='directory', default='.',
            help='Output directory of converted files.'),
    )
    
    def handle(self, options, global_options, *args):
        from docutils.core import publish_file
        import glob
        from . import markdown_writer
        from .utils import walk_dirs
        
        source = os.getcwd()
        
        if not os.path.exists(options.directory):
            os.makedirs(options.directory)
            
        if not args:
            files = walk_dirs(source, include_ext=options.extension, file_only=True)
        else:
            files = args
        for f in files:
            nf = os.path.join(options.directory, os.path.splitext(os.path.basename(f))[0] + '.md')
            if global_options.verbose:
                print('Convert %s...' % f)
            publish_file(source_path=f, 
                destination_path=nf,
                writer=markdown_writer.Writer())
        
register_command(Rst2MdCommand)

def main():
    if conf:
        modules = getattr(conf, 'plugins', [])
    else:
        modules = []
    call('parm', __version__, modules)
    
if __name__ == '__main__':
    main()
